@@ -7,8 +7,8 @@ expect_error(flexsurvreg(formula = Surv(ovarian$futime, ovarian$fustat) ~ 1, dat
 
 test_that("Generalized gamma fit",{
     fitg <- flexsurvreg(formula = Surv(ovarian$futime, ovarian$fustat) ~ 1, dist="gengamma")
-    print(fitg)
-    print(fitg, digits=4)
+    # print(fitg)
+    # print(fitg, digits=4)
     fitg <- flexsurvreg(formula = Surv(futime, fustat) ~ 1, data = ovarian, dist="gengamma")
     ovarian2 <- ovarian
     fitg <- flexsurvreg(formula = Surv(futime, fustat) ~ 1, data = ovarian2, dist="gengamma")
@@ -171,7 +171,6 @@ test_that("Summary function: alternative ways to supply covariates",{
 
 test_that("summary with CIs",{
     summ <- summary(fitg, newdata=data.frame(rx=1), B=2, type="survival")
-    print(summ)
     expect_true(all(unlist(summ[[1]][,2:4]) <= 1))
     expect_true(all(unlist(summ[[1]][,2:4]) >= 0))
 })
@@ -215,14 +214,17 @@ test_that("Model fit with covariates and simulated data",{
     sim <- rgengamma(500, 1.5 + 2*x3, 1, -0.4)
     dead <- as.numeric(sim<=30)
     simt <- ifelse(sim<=30, sim, 30)
-    fit <- flexsurvreg(Surv(simt, dead) ~ x3, dist="gengamma", control=list(maxit=10000))
-    fit <- flexsurvreg(Surv(simt, dead) ~ x + x2 + x3, dist="gengamma", control=list(maxit=10000))
-    fit <- flexsurvreg(Surv(simt, dead)[1:100] ~ x[1:100] + x2[1:100], dist="gengamma", control=list(maxit=10000), method="BFGS")
-    fit <- flexsurvreg(Surv(simt, dead)[1:100] ~ x[1:100], dist="gengamma", control=list(maxit=10000))
-    fit
+    expect_error({
+      fit <- flexsurvreg(Surv(simt, dead) ~ x3, dist="gengamma", control=list(maxit=10000))
+      fit <- flexsurvreg(Surv(simt, dead) ~ x + x2 + x3, dist="gengamma", control=list(maxit=10000))
+      fit <- flexsurvreg(Surv(simt, dead)[1:100] ~ x[1:100] + x2[1:100], dist="gengamma", control=list(maxit=10000), method="BFGS")
+      fit <- flexsurvreg(Surv(simt, dead)[1:100] ~ x[1:100], dist="gengamma", control=list(maxit=10000))
+    }, NA)
 })
 
 test_that("Covariates on ancillary parameters",{
+  
+  expect_error({
     set.seed(11082012)
     x3 <- rnorm(1500,0,1)
     x4 <- rnorm(1500,0,1)
@@ -231,25 +233,22 @@ test_that("Covariates on ancillary parameters",{
     dead <- as.numeric(sim<=30)
     simt <- ifelse(sim<=30, sim, 30)
 
-    fit <- flexsurvreg(Surv(simt, dead) ~ sigma(x3), dist="gengamma", control=list(maxit=10000))
-    fit
-    cl <- confint(fit)
-
     ## Cov on ancillary, not on location
-    flexsurvreg(Surv(simt, dead) ~ sigma(x3), dist="gengamma")
-    flexsurvreg(Surv(simt, dead) ~ 1, anc=list(sigma=~x3), dist="gengamma")
+    flexsurvreg(Surv(simt, dead) ~ sigma(x3), dist="gengamma", fixedpars=TRUE)
+    flexsurvreg(Surv(simt, dead) ~ 1, anc=list(sigma=~x3), dist="gengamma", fixedpars=TRUE)
 
     ## Cov on both location and ancillary
-    flexsurvreg(Surv(simt, dead) ~ x3 + sigma(x3), dist="gengamma")
-    flexsurvreg(Surv(simt, dead) ~ x3, anc=list(sigma=~x3), dist="gengamma")
+    flexsurvreg(Surv(simt, dead) ~ x3 + sigma(x3), dist="gengamma", fixedpars=TRUE)
+    flexsurvreg(Surv(simt, dead) ~ x3, anc=list(sigma=~x3), dist="gengamma", fixedpars=TRUE)
 
     ## More than one covariate on an ancillary parameter
-    flexsurvreg(Surv(simt, dead) ~ x3 + sigma(x3) + sigma(x4), dist="gengamma")
-    flexsurvreg(Surv(simt, dead) ~ x3, anc=list(sigma=~x3+x4), dist="gengamma")
+    flexsurvreg(Surv(simt, dead) ~ x3 + sigma(x3) + sigma(x4), dist="gengamma", fixedpars=TRUE)
+    flexsurvreg(Surv(simt, dead) ~ x3, anc=list(sigma=~x3+x4), dist="gengamma", fixedpars=TRUE)
 
     ## More than one ancillary parameter with covariates
-    flexsurvreg(Surv(simt, dead) ~ x3 + sigma(x3) + sigma(x4) + Q(x5), dist="gengamma")
-    x <- flexsurvreg(Surv(simt, dead) ~ x3, anc=list(sigma=~x3+x4, Q=~x5), dist="gengamma")
+    flexsurvreg(Surv(simt, dead) ~ x3 + sigma(x3) + sigma(x4) + Q(x5), dist="gengamma", fixedpars=TRUE)
+    x <- flexsurvreg(Surv(simt, dead) ~ x3, anc=list(sigma=~x3+x4, Q=~x5), dist="gengamma", fixedpars=TRUE)
+  }, NA)
 })
 
 test_that("Various errors",{
@@ -277,15 +276,19 @@ test_that("Various errors",{
 })
 
 test_that("Calling flexsurvreg from within a function",{
+  
+  expect_error({
     f <- function(){
         ovarian2 <- ovarian
         fitg <- flexsurvreg(formula = Surv(futime, fustat) ~ 1, data = ovarian2, dist="gengamma")
         fitg <- flexsurvreg(formula = Surv(ovarian2$futime, ovarian2$fustat) ~ factor(ovarian2$rx), dist="gengamma",method="Nelder-Mead")
     }
     f()
+  }, NA)
 })
 
 test_that("Calling flexsurvreg from a function within a function",{
+  expect_error({
     f <- function(){
         ovarian2 <- ovarian
         g <- function(){
@@ -295,6 +298,7 @@ test_that("Calling flexsurvreg from a function within a function",{
         g()
     }
     f()
+  }, NA)
 })
 
 ## Left-truncation.
@@ -350,6 +354,7 @@ test_that("Interval censoring",{
     fs3 <- flexsurvreg(Surv(tmin, tmax, status, type="interval") ~ 1, dist="weibull")
     expect_equal(fs1$loglik, fs3$loglik)
 
+    
 })
 
 test_that("inits",{
@@ -395,10 +400,10 @@ test_that("Relative survival", {
                                  c(1, median(t[t>0]) / log(2))
                              })
 
-#     fs6b <- flexsurvreg(Surv(recyrs, censrec) ~ group, data=bc, dist=custom.weibullPH, bhazard=bh, dfns=list(h=hweibullPH, H=HweibullPH), fixedpars=TRUE)
-
-    fs6b <- flexsurvreg(Surv(recyrs, censrec) ~ group, data=bc, dist=custom.weibullPH, bhazard=bh, dfns=list(h=hweibullPH, H=HweibullPH))
-    fs6bd <- flexsurvreg(Surv(recyrs, censrec) ~ group, data=bc, dist="weibullPH", bhazard=bh)
+    suppressWarnings({  # warnings from temporary overflow during optimisation 
+      fs6b <- flexsurvreg(Surv(recyrs, censrec) ~ group, data=bc, dist=custom.weibullPH, bhazard=bh, dfns=list(h=hweibullPH, H=HweibullPH))
+      fs6bd <- flexsurvreg(Surv(recyrs, censrec) ~ group, data=bc, dist="weibullPH", bhazard=bh)
+    })
     
     expect_equal(log(fs6b$res[1,"est"]), 0.3268327417773233, tol=1e-05)
     expect_equal(log(fs6b$res[2,"est"]), -3.5308925743338038, tol=1e-05)
@@ -435,61 +440,6 @@ test_that("Weibull hazards from summary are reliable",{
   expect_true(all(is.finite(output$est)))
 })
 
-test_that("RMST/Mean/Median calculations are working",{
-  
-  fs1 = flexsurvreg(Surv(rectime, censrec)~group ,dist="weibull",data=bc)
-  fs2 = flexsurvreg(Surv(rectime, censrec)~group ,dist="exp",data=bc)
-  fs3 = flexsurvreg(Surv(rectime, censrec)~group ,dist="llogis",data=bc)
-  fs4 = flexsurvreg(Surv(rectime, censrec)~group ,dist="lnorm",data=bc)
-  fs5 = flexsurvreg(Surv(rectime, censrec)~group ,dist="gamma",data=bc)
-  
-  res1 = summary(fs1,t=c(Inf),start=0,type="rmst")
-  res2 = summary(fs1,type="mean")
-  
-  res1_len = length(res1)
-  for(i in seq_len(res1_len)){
-    expect_equal(
-      res1[[i]]$est,
-      res2[[i]]$est,
-      tolerance=1e-3
-    )
-  }
-  
-  # Exponential analytical RMST should be consistent w/ analytical
-  # mean.
-  expect_equal(summary(fs2,type="mean",tidy=T)$est,summary(fs2,t=Inf,type="rmst",tidy=T)$est, tolerance=1e-3)
-  
-  # Analytical mean should closely match result from integration
-  expect_equal(summary(fs1,type="mean",tidy=T)$est,summary(fs1,t=Inf,type="rmst",tidy=T)$est, tolerance=1e-3)
-  expect_equal(summary(fs3,type="mean",tidy=T)$est,summary(fs3,t=Inf,type="rmst",tidy=T)$est, tolerance=1e-3)
-  expect_equal(summary(fs4,type="mean",tidy=T)$est,summary(fs4,t=Inf,type="rmst",tidy=T)$est, tolerance=1e-3)
-  expect_equal(summary(fs5,type="mean",tidy=T)$est,summary(fs5,t=Inf,type="rmst",tidy=T)$est, tolerance=1e-3)
-  
-  
-  # RMST of exponential to 100 starting at 0 should be the same
-  # as RMST to 200 starting at 100.
-  res3 = summary(fs2,t=c(100,200),start=c(0,100),type="rmst")
-  res3_len = length(res3)
-  for(i in seq_len(res1_len)){
-    expect_equal(
-      res3[[i]]$est[1],
-      res3[[i]]$est[2],
-      tolerance=1e-3
-    )
-  }
-  
-  expect_warning(
-    summary(fs1,t=10,type="mean"),
-    "Mean selected, but time specified.  For restricted mean, set type to 'rmst'."
-  )
-  
-  expect_warning(
-    summary(fs1,t=10,type="median"),
-    "Median selected, but time specified."
-  )
-  
-})
-
 test_that("No events in the data",{
     set.seed(1)
     tmin <- rexp(100, 1) 
